; Example Program glcd01.a51
; Graphic LCD: Solomon LM6321 (64 Columns x 8 Rows x 8 Bits x 2 Panels)
; Chinese Character
; Chin-Shiuh Shieh
; 2000-03-23
;
; Ping Assignment
GLCD_DI		EQU		P2.0	; 1 for Data, 0 for Instruction
GLCD_RW		EQU		P2.1	; 1 for Read, 0 for Write
GLCD_E		EQU		P2.2	; LCD Enable, 1 for Read, 1->0 for Write
GLCD_CS1	EQU		P2.3	; 1 for Chip Select 1 (Left Panel)
GLCD_CS2	EQU		P2.4	; 1 for Chip Select 2 (Right Panel)
GLCD_RST	EQU		P2.5	; 0 for LCD Reset
GLCD_PORT	EQU		P1		; LCD Port
; Memory Allocation
X       	DATA 	30h		; X-coordinate, vertically from 0(top) to 63(down)
Y       	DATA 	31h		; Y-coordinate, horizontally from 0(left) to 127(right)
XP      	DATA 	32h		; X-Page, vertically from 10111000b(top) to 10111111b(down)
YA      	DATA 	33h		; Y-Address, horizontally from 01000000b(left) to 01111111b(right)
MASK    	DATA 	34h		; LCD data pattern
VALUE   	DATA 	35h		; LCD data
;
PROG	EQU		0000h
		ORG		PROG+0000h
		SJMP	START
;
;Interrupt Vector Table
		ORG		PROG+0003h
		LCALL	INT0_ISR
		RETI
		ORG		PROG+000Bh
		LCALL	T0_ISR
		RETI
		ORG		PROG+0013h
		LCALL	INT1_ISR
		RETI
		ORG		PROG+001Bh
		LCALL	T1_ISR
		RETI
		ORG		PROG+0023h
		LCALL	UART_ISR
		RETI
;
		ORG		PROG+0030h
START:
		CALL	GLCD_INIT
        MOV     DPTR,#SHIEH
		SETB	GLCD_CS1
        MOV     XP,#10111000b
LOOP1:
		MOV		A,XP
		CALL	GLCD_WI
        MOV     YA,#01000000b
		MOV		A,YA
		CALL	GLCD_WI
LOOP2:
		MOV		A,#00h
		MOVC	A,@A+DPTR
		CALL	GLCD_WD
		INC		DPTR
        INC     YA
        MOV     A,YA
        CJNE    A,#01100000b,LOOP2
        INC     XP
        MOV     A,XP
        CJNE    A,#10111100b,LOOP1
        CLR		GLCD_CS1
;
        MOV     DPTR,#CHIN
		SETB	GLCD_CS1
        MOV     XP,#10111000b
LOOP3:
		MOV		A,XP
		CALL	GLCD_WI
        MOV     YA,#01100000b
		MOV		A,YA
		CALL	GLCD_WI
LOOP4:
		MOV		A,#00h
		MOVC	A,@A+DPTR
		CALL	GLCD_WD
		INC		DPTR
        INC     YA
        MOV     A,YA
        CJNE    A,#10000000b,LOOP4
        INC     XP
        MOV     A,XP
        CJNE    A,#10111100b,LOOP3
        CLR		GLCD_CS1
;
        MOV     DPTR,#SHIUH
		SETB	GLCD_CS2
        MOV     XP,#10111000b
LOOP5:
		MOV		A,XP
		CALL	GLCD_WI
        MOV     YA,#01000000b
		MOV		A,YA
		CALL	GLCD_WI
LOOP6:
		MOV		A,#00h
		MOVC	A,@A+DPTR
		CALL	GLCD_WD
		INC		DPTR
        INC     YA
        MOV     A,YA
        CJNE    A,#01100000b,LOOP6
        INC     XP
        MOV     A,XP
        CJNE    A,#10111100b,LOOP5
        CLR		GLCD_CS2
DONE:	SJMP	DONE
;
INT0_ISR:
		RET
T0_ISR:
		RET
INT1_ISR:
		RET
T1_ISR:
		RET
;
UART_ISR:
		CLR		TI
		CLR		RI
		RET
;
GLCD_WAIT:
		PUSH	ACC
		MOV		GLCD_PORT,#0FFh	; Reset GLCD_Port for input
		CLR		GLCD_DI
		SETB	GLCD_RW
GLCD_BUSY:
		SETB	GLCD_E
		MOV		A,GLCD_PORT
		CLR		GLCD_E
		JB		ACC.7,GLCD_BUSY
		POP		ACC
		RET
;
GLCD_WI:
		CALL	GLCD_WAIT
		CLR		GLCD_DI
		CLR		GLCD_RW
		MOV		GLCD_PORT,A
		SETB	GLCD_E
		CLR		GLCD_E
		RET
;
GLCD_WD:
		CALL	GLCD_WAIT
		SETB	GLCD_DI
		CLR		GLCD_RW
		MOV		GLCD_PORT,A
		SETB	GLCD_E
		CLR		GLCD_E
		RET
;
GLCD_RD:
        CALL    GLCD_WAIT
		MOV		GLCD_PORT,#0FFh		; Reset GLCD_Port for input
        SETB    GLCD_DI
		SETB	GLCD_RW
		SETB	GLCD_E
        MOV     A,GLCD_PORT
		CLR		GLCD_E
		RET
;
GLCD_INIT:
		CLR		GLCD_RST			; Reset LM6321
		SETB	GLCD_RST
		MOV		GLCD_PORT,#0FFh		; Reset GLCD_Port for input
; Reset left panel chip
		SETB	GLCD_CS1
		CLR		GLCD_DI
		SETB	GLCD_RW
GLCD_INIT1:
		SETB	GLCD_E
		MOV		A,GLCD_PORT
		CLR		GLCD_E
		JB		ACC.4,GLCD_INIT1	; Wait for left panel chip reset
		CLR		GLCD_CS1
; Reset right panel chip
		SETB	GLCD_CS2
		CLR		GLCD_DI
		SETB	GLCD_RW
GLCD_INIT2:
		SETB	GLCD_E
		MOV		A,GLCD_PORT
		CLR		GLCD_E
		JB		ACC.4,GLCD_INIT2	; Wait for right panel chip reset
		CLR		GLCD_CS2
; Clear screen
		CALL	GLCD_CLS
; Turn on left panel
		SETB	GLCD_CS1
		MOV		A,#00111111b		; Turn on LCD
		CALL	GLCD_WI
		MOV		A,#10111000b		; Set X-Page to 0
		CALL	GLCD_WI
		MOV		A,#01000000b		; Set Y-Address to 0
		CALL	GLCD_WI
		MOV		A,#11000000b		; Set display start line to 0
		CALL	GLCD_WI
		CLR		GLCD_CS1
; Turn on right panel
		SETB	GLCD_CS2
		MOV		A,#00111111b		; Turn on LCD
		CALL	GLCD_WI
		MOV		A,#10111000b		; Set X-page to 0
		CALL	GLCD_WI
		MOV		A,#01000000b		; Set Y-address to 0
		CALL	GLCD_WI
		MOV		A,#11000000b		; Set display start line to 0
		CALL	GLCD_WI
		CLR		GLCD_CS2
		RET
;
GLCD_CLS:
        PUSH    ACC
; Clear left panel
		SETB	GLCD_CS1
        MOV     XP,#10111000b
GLCD_CLS1:
		MOV		A,XP
		CALL	GLCD_WI
        MOV     YA,#01000000b
		MOV		A,YA
		CALL	GLCD_WI
GLCD_CLS2:
		MOV		A,#00h
		CALL	GLCD_WD
        INC     YA
        MOV     A,YA
        CJNE    A,#10000000b,GLCD_CLS2
        INC     XP
        MOV     A,XP
        CJNE    A,#11000000b,GLCD_CLS1
        CLR		GLCD_CS1
; Clear right panel
		SETB	GLCD_CS2
        MOV     XP,#10111000b
GLCD_CLS3:
		MOV		A,XP
		CALL	GLCD_WI
        MOV     YA,#01000000b
		MOV		A,YA
		CALL	GLCD_WI
GLCD_CLS4:
		MOV		A,#00h
		CALL	GLCD_WD
        INC     YA
        MOV     A,YA
        CJNE    A,#10000000b,GLCD_CLS4
        INC     XP
        MOV     A,XP
        CJNE    A,#11000000b,GLCD_CLS3
        CLR		GLCD_CS2
		POP     ACC
        RET
;
; Coordinate Transform
; X-Page, Y-Address, Mask calculation for given X, Y coordinates
GLCD_CT:
; Calculate X-Page
        MOV     A,X
        RR      A
        RR      A
        RR      A
        ORL     A,#10111000b
        ANL     A,#10111111b
        MOV     XP,A
; Calculate Chip and Y-Address
        MOV     A,Y
		CLR		C
		SUBB	A,#64
		JC		GLCD_CT4
        CLR     GLCD_CS1
		SETB	GLCD_CS2
		SJMP	GLCD_CT5
GLCD_CT4:
		MOV		A,Y
		SETB	GLCD_CS1
		CLR		GLCD_CS2
GLCD_CT5:
        ORL     A,#01000000b
        ANL     A,#01111111b
        MOV     YA,A
; Calculate bit mask
        MOV     A,X
		ANL		A,#00000111b
		MOV		R0,A
		MOV		A,#00000001b
GLCD_CT1:
        CJNE    R0,#00h,GLCD_CT2
		SJMP	GLCD_CT3
GLCD_CT2:
		RL		A
        DEC     R0
		SJMP	GLCD_CT1
GLCD_CT3:
        MOV     MASK,A
		RET
;
; Set a pixel
GLCD_SET:
        CALL    GLCD_CT
        MOV     A,XP
        CALL    GLCD_WI
        MOV     A,YA
        CALL    GLCD_WI
        CALL    GLCD_RD		; Dummy read
        CALL    GLCD_RD		; Actual read
        MOV     VALUE,A
        MOV     A,MASK
        ORL     A,VALUE
        MOV     VALUE,A
        MOV     A,XP
        CALL    GLCD_WI
        MOV     A,YA
        CALL    GLCD_WI
        MOV     A,VALUE
        CALL    GLCD_WD
        RET
;
; Clear a pixel
GLCD_CLR:
        CALL    GLCD_CT
        MOV     A,XP
        CALL    GLCD_WI
        MOV     A,YA
        CALL    GLCD_WI
        CALL    GLCD_RD		; Dummy read
        CALL    GLCD_RD		; Actual read
        MOV     VALUE,A
        MOV     A,MASK
        XRL     A,#11111111b
        ANL     A,VALUE
        MOV     VALUE,A
        MOV     A,XP
        CALL    GLCD_WI
        MOV     A,YA
        CALL    GLCD_WI
        MOV     A,VALUE
        CALL    GLCD_WD
        RET
;
; Negate a pixel
GLCD_XOR:
        CALL    GLCD_CT
        MOV     A,XP
        CALL    GLCD_WI
        MOV     A,YA
        CALL    GLCD_WI
        CALL    GLCD_RD		; Dummy read
        CALL    GLCD_RD		; Actual read
        MOV     VALUE,A
        MOV     A,MASK
        XRL     A,VALUE
        MOV     VALUE,A
        MOV     A,XP
        CALL    GLCD_WI
        MOV     A,YA
        CALL    GLCD_WI
        MOV     A,VALUE
        CALL    GLCD_WD
        RET
;
SHIEH:
	DB	000h,000h,000h,000h,000h,000h,002h,082h
	DB	086h,0C6h,0C6h,046h,040h,040h,0E0h,0F8h
	DB	05Fh,07Fh,0F4h,0F0h,040h,000h,000h,002h
	DB	003h,0FFh,0FFh,086h,0C0h,0C0h,040h,000h
	DB	004h,004h,006h,006h,007h,013h,091h,0D9h
	DB	0CCh,0ECh,064h,064h,000h,0FEh,0FFh,0F3h
	DB	092h,0D2h,0FFh,0FFh,0F0h,060h,030h,018h
	DB	00Ch,0FFh,0FFh,003h,000h,000h,000h,000h
	DB	000h,000h,000h,040h,0F9h,0FFh,01Fh,0DDh
	DB	0EEh,0FEh,07Eh,03Eh,0D6h,0F6h,0FBh,03Fh
	DB	01Fh,00Fh,0FFh,0FFh,000h,005h,007h,007h
	DB	002h,0FFh,0FFh,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,001h,001h,001h,001h
	DB	001h,000h,000h,000h,000h,000h,000h,000h
	DB	001h,003h,00Fh,00Fh,00Ch,001h,003h,006h
	DB	00Eh,01Fh,01Fh,00Fh,000h,000h,000h,000h
CHIN:
	DB	000h,000h,000h,000h,000h,000h,000h,080h
	DB	0F0h,0FEh,07Eh,01Eh,050h,070h,030h,030h
	DB	020h,000h,084h,0FCh,0F8h,000h,000h,000h
	DB	080h,080h,080h,000h,000h,000h,000h,000h
	DB	000h,080h,0C0h,060h,038h,01Ch,00Fh,007h
	DB	08Fh,0CCh,0F6h,07Eh,0BAh,092h,080h,000h
	DB	018h,03Ch,0BFh,0DFh,0CFh,0C6h,003h,021h
	DB	031h,03Fh,01Fh,00Fh,002h,000h,000h,000h
	DB	001h,001h,000h,000h,008h,019h,011h,019h
	DB	00Dh,0BFh,0FFh,064h,037h,013h,003h,000h
	DB	080h,0C0h,0E0h,0F8h,07Fh,00Fh,010h,030h
	DB	070h,0E0h,0E0h,0C0h,080h,080h,000h,000h
	DB	000h,000h,000h,000h,000h,002h,002h,003h
	DB	003h,003h,001h,000h,000h,000h,000h,001h
	DB	001h,000h,000h,000h,000h,000h,000h,000h
    DB  000h,000h,001h,003h,007h,007h,000h,000h
SHIUH:
	DB	000h,000h,000h,000h,000h,040h,0C0h,0C0h
	DB	080h,000h,000h,080h,080h,000h,000h,000h
	DB	000h,000h,080h,0C0h,040h,040h,0C0h,0C0h
	DB	080h,000h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,010h,010h,010h,098h,0FFh,0FFh
	DB	07Fh,006h,0C3h,0FFh,0FFh,00Fh,002h,000h
	DB	0FEh,0FEh,0F8h,038h,038h,018h,0FFh,0FFh
	DB	0FFh,000h,000h,000h,000h,000h,000h,000h
	DB	000h,080h,0E0h,070h,03Ch,01Fh,007h,001h
	DB	000h,000h,01Fh,07Fh,0E0h,0C0h,080h,000h
	DB	003h,003h,003h,003h,003h,007h,007h,087h
	DB	083h,080h,0C0h,0C0h,0F4h,0FCh,078h,000h
	DB	001h,001h,000h,000h,000h,000h,000h,000h
	DB	000h,000h,000h,000h,000h,000h,001h,001h
	DB	001h,001h,003h,003h,003h,003h,001h,001h
	DB	001h,001h,001h,001h,000h,000h,000h,000h
;
		END